@@ -181,7 +181,7 @@
              [(eof-object? c)
               (if (null? sofar)
                   #f
-                  sofar)]
+                  (reverse sofar))]
              [(equal? #\newline c)
               (reverse sofar)]
              [else (loop (cons c sofar))])))
