@@ -38,6 +38,8 @@ inline static void BTC_register_new_thread(void *t, void *c)
 }
 
 inline static void BTC_register_thread(void *t, void *c)
+/* Might be called in a future thread to change to a custodian that
+   has a set number */
 {
   NewGC *gc = GC_get_GC();
   GC_Thread_Info *work;
@@ -88,7 +90,7 @@ inline static void clean_up_thread_list(NewGC *gc)
   GC_Thread_Info *prev = NULL;
 
   while(work) {
-    if(!pagemap_find_page(gc->page_maps, work->thread) || marked(gc, work->thread)) {
+    if (marked(gc, work->thread)) {
       work->thread = GC_resolve2(work->thread, gc);
       prev = work;
       work = work->next;
@@ -136,7 +138,8 @@ inline static int create_blank_owner_set(NewGC *gc)
   gc->owner_table_size = curr_size;
 
   naya = (OTEntry **)ofm_malloc(curr_size * sizeof(OTEntry*));
-  memcpy(naya, owner_table, old_size*sizeof(OTEntry*));
+  if (old_size)
+    memcpy(naya, owner_table, old_size*sizeof(OTEntry*));
   gc->owner_table = owner_table = naya;
   bzero(((char*)owner_table) + (sizeof(OTEntry*) * old_size),
         (curr_size - old_size) * sizeof(OTEntry*));
@@ -144,10 +147,12 @@ inline static int create_blank_owner_set(NewGC *gc)
   return create_blank_owner_set(gc);
 }
 
-inline static int custodian_to_owner_set(NewGC *gc,Scheme_Custodian *cust)
+inline static int custodian_to_owner_set(NewGC *gc, Scheme_Custodian *cust)
 {
   int i;
 
+  GC_ASSERT(SAME_TYPE(SCHEME_TYPE(cust), scheme_custodian_type));
+
   if (cust->gc_owner_set)
     return cust->gc_owner_set;
 
@@ -223,21 +228,27 @@ inline static void clean_up_owner_table(NewGC *gc)
 {
   OTEntry **owner_table = gc->owner_table;
   const int table_size = gc->owner_table_size;
-  int i;
+  int i, really_doing_accounting = 0;
 
   for(i = 1; i < table_size; i++)
     if(owner_table[i]) {
       /* repair or delete the originator */
       if(!marked(gc, owner_table[i]->originator)) {
         owner_table[i]->originator = NULL;
-      } else 
+      } else {
         owner_table[i]->originator = GC_resolve2(owner_table[i]->originator, gc);
+        if (((Scheme_Custodian *)owner_table[i]->originator)->really_doing_accounting) {
+          really_doing_accounting = 1;
+        }
+      }
 
       /* potential delete */
       if(i != 1) 
         if((owner_table[i]->memory_use == 0) && !owner_table[i]->originator)
           free_owner_set(gc, i);
     }
+
+  gc->next_really_doing_accounting |= really_doing_accounting;
 }
 
 inline static uintptr_t custodian_usage(NewGC*gc, void *custodian)
@@ -246,11 +257,13 @@ inline static uintptr_t custodian_usage(NewGC*gc, void *custodian)
   uintptr_t retval = 0;
   int i;
 
+  ((Scheme_Custodian *)custodian)->really_doing_accounting = 1;
+
   if(!gc->really_doing_accounting) {
     if (!gc->avoid_collection) {
       CHECK_PARK_UNUSED(gc);
       gc->park[0] = custodian;
-      gc->really_doing_accounting = 1;
+      gc->next_really_doing_accounting = 1;
       garbage_collect(gc, 1, 0, 0, NULL);
       custodian = gc->park[0]; 
       gc->park[0] = NULL;
@@ -356,7 +369,7 @@ inline static void BTC_memory_account_mark(NewGC *gc, mpage *page, void *ptr, in
       if(info->btc_mark == gc->old_btc_mark) {
         info->btc_mark = gc->new_btc_mark;
         account_memory(gc, gc->current_mark_owner, gcBYTES_TO_WORDS(page->size), is_a_master_page);
-        push_ptr(gc, TAG_AS_BIG_PAGE_PTR(ptr));
+        push_ptr(gc, TAG_AS_BIG_PAGE_PTR(ptr), 0);
       }
     } else {
       /* medium page */
@@ -366,7 +379,7 @@ inline static void BTC_memory_account_mark(NewGC *gc, mpage *page, void *ptr, in
         info->btc_mark = gc->new_btc_mark;
         account_memory(gc, gc->current_mark_owner, info->size, is_a_master_page);
         ptr = OBJHEAD_TO_OBJPTR(info);
-        push_ptr(gc, ptr);
+        push_ptr(gc, ptr, 0);
       }
     }
   } else {
@@ -375,7 +388,8 @@ inline static void BTC_memory_account_mark(NewGC *gc, mpage *page, void *ptr, in
     if(info->btc_mark == gc->old_btc_mark) {
       info->btc_mark = gc->new_btc_mark;
       account_memory(gc, gc->current_mark_owner, info->size, 0);
-      push_ptr(gc, ptr);
+      if (page->generation != AGE_GEN_HALF)
+        push_ptr(gc, ptr, 0);
     }
   }
 }
@@ -460,15 +474,34 @@ static void btc_overmem_abort(NewGC *gc)
         "Info will be wrong.\n"));
 }
 
-static void propagate_accounting_marks(NewGC *gc)
+static void propagate_accounting_marks(NewGC *gc, int no_full)
 {
   void *p;
+  int fuel = (gc->gc_full
+              ? -1
+              : (no_full
+                 ? INCREMENTAL_COLLECT_FUEL_PER_100M / INCREMENTAL_MINOR_REQUEST_DIVISOR
+                 : (INCREMENTAL_COLLECT_FUEL_PER_100M * AS_100M(gc->memory_in_use)) / 2));
 
-  while(pop_ptr(gc, &p) && !gc->kill_propagation_loop) {
+  while (pop_ptr(gc, &p, 0) && !gc->kill_propagation_loop) {
+    gc->traverse_count = 0;
+    
     /* GCDEBUG((DEBUGOUTF, "btc_account: popped off page %p:%p, ptr %p\n", page, page->addr, p)); */
-    propagate_marks_worker(gc, p);
+    propagate_marks_worker(gc, p, 0);
+    
+    if (fuel >= 0) {
+      fuel--;
+      fuel -= (gc->traverse_count >> 2);
+      if (gc->unprotected_page) {
+        gc->unprotected_page = 0;
+        fuel -= 100;
+      }
+      if (fuel <= 0)
+        break;
+    }
   }
-  if(gc->kill_propagation_loop)
+
+  if (gc->kill_propagation_loop)
     reset_pointer_stack(gc);
 }
 
@@ -489,24 +522,71 @@ inline static int BTC_get_redirect_tag(NewGC *gc, int tag) {
   return tag;
 }
 
-static void BTC_do_accounting(NewGC *gc)
+static void BTC_do_accounting(NewGC *gc, int no_full)
 {
   const int table_size = gc->owner_table_size;
+  int init_table_start, init_table_end, do_mark_threads;
   OTEntry **owner_table = gc->owner_table;
+  MarkSegment *orig_mark_stack;
 
+  GC_ASSERT(gc->gc_full || gc->finished_incremental);
+  GC_ASSERT(gc->gc_full || !gc->accounted_incremental);
+
+  if (gc->gc_full) {
+    if (!gc->acct_mark_stack)
+      gc->really_doing_accounting = gc->next_really_doing_accounting;
+    gc->next_really_doing_accounting = 0;
+  } else {
+    if (gc->next_really_doing_accounting)
+      gc->really_doing_accounting = 1;
+
+    GC_ASSERT(!gc->mark_gen1);
+    GC_ASSERT(!gc->inc_gen1);
+    GC_ASSERT(!gc->check_gen1);
+    
+    gc->mark_gen1 = 1;
+    gc->check_gen1 = 1;
+    gc->inc_gen1 = 1;
+  }
+  
   if(gc->really_doing_accounting) {
     Scheme_Custodian *cur = owner_table[current_owner(gc, NULL)]->originator, *last, *parent;
     Scheme_Custodian_Reference *box = cur->global_next;
     int i;
 
+    GC_ASSERT(SAME_TYPE(SCHEME_TYPE(cur), scheme_custodian_type));
+
     GCDEBUG((DEBUGOUTF, "\nBEGINNING MEMORY ACCOUNTING\n"));
     gc->doing_memory_accounting = 1;
     gc->in_unsafe_allocation_mode = 1;
     gc->unsafe_allocation_abort = btc_overmem_abort;
     gc->master_page_btc_mark_checked = 0;
 
+    if (!gc->gc_full || gc->acct_mark_stack) {
+      orig_mark_stack = gc->mark_stack;
+      if (gc->acct_mark_stack) {
+        gc->mark_stack = gc->acct_mark_stack;
+        init_table_start = 2;
+        do_mark_threads = 0;
+      } else {
+        gc->mark_stack = NULL;
+        mark_stack_initialize(gc);
+        init_table_start = 1;
+        do_mark_threads = 1;
+      }
+      if (gc->gc_full)
+        init_table_end = table_size;
+      else
+        init_table_end = 2;
+    } else {
+      orig_mark_stack = NULL;
+      init_table_start = 1;
+      init_table_end = table_size;
+      do_mark_threads = 1;
+    }
+
     /* clear the memory use numbers out */
-    for(i = 1; i < table_size; i++)
+    for(i = init_table_start; i < init_table_end; i++)
       if(owner_table[i]) {
         owner_table[i]->memory_use = 0;
 #ifdef MZ_USE_PLACES
@@ -514,64 +594,93 @@ static void BTC_do_accounting(NewGC *gc)
           owner_table[i]->master_memory_use = 0;
 #endif
       }
-    
+
     /* start with root: */
     while (cur->parent && SCHEME_PTR1_VAL(cur->parent)) {
       cur = SCHEME_PTR1_VAL(cur->parent);
+      GC_ASSERT(SAME_TYPE(SCHEME_TYPE(cur), scheme_custodian_type));
     }
 
     /* walk forward for the order we want (blame parents instead of children) */
     last = cur;
     while(cur) {
       int owner = custodian_to_owner_set(gc, cur);
-      uintptr_t save_count = gc->phantom_count;
 
-      gc->phantom_count = 0;
+      GC_ASSERT(gc->gc_full || (owner == 1));
+      
+      GC_ASSERT(owner >= 0);
+      GC_ASSERT(owner <= gc->owner_table_size);
+      
+      gc->acct_phantom_count = 0;
 
       gc->current_mark_owner = owner;
       GCDEBUG((DEBUGOUTF,"MARKING THREADS OF OWNER %i (CUST %p)\n", owner, cur));
       gc->kill_propagation_loop = 0;
-      mark_threads(gc, owner);
-      mark_cust_boxes(gc, cur);
+      if (do_mark_threads)  {
+        mark_threads(gc, owner);
+        mark_cust_boxes(gc, cur);
+      }
       GCDEBUG((DEBUGOUTF, "Propagating accounting marks\n"));
-      propagate_accounting_marks(gc);
-
-      last = cur;
-      box = cur->global_next; cur = box ? SCHEME_PTR1_VAL(box) : NULL;
+      propagate_accounting_marks(gc, no_full);
 
       owner_table = gc->owner_table;
       owner_table[owner]->memory_use = add_no_overflow(owner_table[owner]->memory_use, 
-                                                       gcBYTES_TO_WORDS(gc->phantom_count));
-      gc->phantom_count = save_count;
+                                                       gcBYTES_TO_WORDS(gc->acct_phantom_count));
+
+      if (!gc->gc_full)
+        break;
+
+      last = cur;
+      box = cur->global_next;
+      cur = box ? SCHEME_PTR1_VAL(box) : NULL;
+
+      do_mark_threads = 1;
     }
 
     release_master_btc_mark(gc);
 
-    /* walk backward folding totals int parent */
-    cur = last;
-    while (cur) {
-      int owner = custodian_to_owner_set(gc, cur);
+    if (gc->gc_full) {
+      /* walk backward folding totals into parent */
+      cur = last;
+      while (cur) {
+        int owner = custodian_to_owner_set(gc, cur);
       
-      box = cur->parent; parent = box ? SCHEME_PTR1_VAL(box) : NULL;
-      if (parent) {
-        int powner = custodian_to_owner_set(gc, parent);
+        box = cur->parent; parent = box ? SCHEME_PTR1_VAL(box) : NULL;
+        if (parent) {
+          int powner = custodian_to_owner_set(gc, parent);
 
-        owner_table = gc->owner_table;
-        owner_table[powner]->memory_use = add_no_overflow(owner_table[powner]->memory_use,
-                                                          owner_table[owner]->memory_use);
-        owner_table[powner]->master_memory_use += owner_table[owner]->master_memory_use;
+          owner_table = gc->owner_table;
+          owner_table[powner]->memory_use = add_no_overflow(owner_table[powner]->memory_use,
+                                                            owner_table[owner]->memory_use);
+          owner_table[powner]->master_memory_use += owner_table[owner]->master_memory_use;
+        }
+
+        box = cur->global_prev; cur = box ? SCHEME_PTR1_VAL(box) : NULL;
       }
 
-      box = cur->global_prev; cur = box ? SCHEME_PTR1_VAL(box) : NULL;
+      if (orig_mark_stack) {
+        free_stack_pages_at(gc->mark_stack);
+        gc->acct_mark_stack = NULL;
+        gc->mark_stack = orig_mark_stack;
+      }
+    } else {
+      gc->acct_mark_stack = gc->mark_stack;
+      gc->mark_stack = orig_mark_stack;
     }
 
     gc->in_unsafe_allocation_mode = 0;
     gc->doing_memory_accounting = 0;
-    gc->old_btc_mark = gc->new_btc_mark;
-    gc->new_btc_mark = !gc->new_btc_mark;
+    if (gc->gc_full) {
+      gc->old_btc_mark = gc->new_btc_mark;
+      gc->new_btc_mark = !gc->new_btc_mark;
+    }
   }
 
-  clear_stack_pages(gc);
+  if (!gc->gc_full) {
+    gc->mark_gen1 = 0;
+    gc->check_gen1 = 0;
+    gc->inc_gen1 = 0;
+  }
 }
 
 inline static void BTC_add_account_hook(int type,void *c1,void *c2,uintptr_t b)
@@ -579,12 +688,14 @@ inline static void BTC_add_account_hook(int type,void *c1,void *c2,uintptr_t b)
   NewGC *gc = GC_get_GC();
   AccountHook *work;
 
+  ((Scheme_Custodian *)c1)->really_doing_accounting = 1;
+
   if(!gc->really_doing_accounting) {
     if (!gc->avoid_collection) {
       CHECK_PARK_UNUSED(gc);
       gc->park[0] = c1; 
       gc->park[1] = c2;
-      gc->really_doing_accounting = 1;
+      gc->next_really_doing_accounting = 1;
       garbage_collect(gc, 1, 0, 0, NULL);
       c1 = gc->park[0]; gc->park[0] = NULL;
       c2 = gc->park[1]; gc->park[1] = NULL;
@@ -769,11 +880,16 @@ int BTC_single_allocation_limit(NewGC *gc, size_t sizeb)
  * GC_out_of_memory protects any user-requested allocation whose size
  * is independent of any existing object, then we can enforce the limit. */
 {
-  Scheme_Thread *p = scheme_current_thread;
-  if (p)
-    return (custodian_single_time_limit(gc, thread_get_owner(p)) < sizeb);
-  else
-    return (gc->place_memory_limit < sizeb);
+  if (gc->alternate_accounting_custodian) {
+    int set = custodian_to_owner_set(gc, gc->alternate_accounting_custodian);
+    return (custodian_single_time_limit(gc, set) < sizeb);
+  } else {
+    Scheme_Thread *p = scheme_current_thread;
+    if (p)
+      return (custodian_single_time_limit(gc, thread_get_owner(p)) < sizeb);
+    else
+      return (gc->place_memory_limit < sizeb);
+  }
 }
 
 static uintptr_t BTC_get_account_hook(void *c1)
