@@ -49,14 +49,12 @@
 #ifdef MZ_PRECISE_GC
 # define MUST_REGISTER_GLOBALS
 # define MZTAG_REQUIRED
-# undef UNIX_IMAGE_DUMPS
 /* In case SGC is used to build PRECISE_GC: */
 # undef USE_SENORA_GC
 #endif
 
 #ifdef USE_SENORA_GC
 # define MUST_REGISTER_GLOBALS
-# undef UNIX_IMAGE_DUMPS
 #endif
 
 #ifdef USE_SINGLE_FLOATS
@@ -153,12 +151,6 @@ typedef long FILE;
 # define MSCBOR_IZE(x) MSC_IZE(x)
 #endif
 
-#ifdef SIGSET_IS_SIGNAL
-# define MZ_SIGSET(s, f) signal(s, f)
-#else
-# define MZ_SIGSET(s, f) sigset(s, f)
-#endif
-
 /* C99 allows an array in a struct to be declared
    with [] to indicate that its actual size can be
    any number. The old way was to declare the array
@@ -213,12 +205,6 @@ typedef jmpbuf jmp_buf[1];
 
 #define GC_MIGHT_USE_REGISTERED_STATICS
 
-#ifdef MACINTOSH_EVENTS
-/* We avoid #including the Carbon headers because we only
-   need a few abstract struct types: */
-typedef struct FSSpec mzFSSpec;
-#endif
-
 #ifndef MZ_DONT_USE_JIT
 # if defined(MZ_USE_JIT_PPC) || defined(MZ_USE_JIT_I386) || defined(MZ_USE_JIT_X86_64) || defined(MZ_USE_JIT_ARM)
 #  define MZ_USE_JIT
@@ -1906,6 +1892,7 @@ MZ_EXTERN void scheme_set_startup_use_jit(int);
 MZ_EXTERN void scheme_set_startup_load_on_demand(int);
 MZ_EXTERN void scheme_set_ignore_user_paths(int);
 MZ_EXTERN void scheme_set_ignore_link_paths(int);
+MZ_EXTERN void scheme_set_cross_compile_mode(int);
 MZ_EXTERN void scheme_set_logging(int syslog_level, int stderr_level);
 MZ_EXTERN void scheme_set_logging_spec(Scheme_Object *syslog_level, Scheme_Object *stderr_level);
 
@@ -1955,9 +1942,6 @@ MZ_EXTERN void (*scheme_suspend_main_thread)(void);
 int scheme_set_in_main_thread(void);
 void scheme_restore_nonmain_thread(void);
 #endif
-#ifdef MAC_FILE_SYSTEM
-extern long scheme_creator_id;
-#endif
 
 typedef Scheme_Object *(*Scheme_Stdio_Maker_Proc)(void);
 MZ_EXTERN Scheme_Object *(*scheme_make_stdin)(void);
@@ -1974,6 +1958,8 @@ MZ_EXTERN Scheme_Object *scheme_set_exec_cmd(char *s);
 MZ_EXTERN Scheme_Object *scheme_set_run_cmd(char *s);
 MZ_EXTERN void scheme_set_collects_path(Scheme_Object *p);
 MZ_EXTERN void scheme_set_config_path(Scheme_Object *p);
+MZ_EXTERN void scheme_set_host_collects_path(Scheme_Object *p);
+MZ_EXTERN void scheme_set_host_config_path(Scheme_Object *p);
 MZ_EXTERN void scheme_set_original_dir(Scheme_Object *d);
 MZ_EXTERN void scheme_set_addon_dir(Scheme_Object *p);
 MZ_EXTERN void scheme_set_command_line_arguments(Scheme_Object *vec);
@@ -2043,6 +2029,12 @@ MZ_EXTERN Scheme_Object *scheme_register_parameter(Scheme_Prim *function, char *
 
 #endif /* SCHEME_DIRECT_EMBEDDED */
 
+/*========================================================================*/
+/*                             OS signals                                 */
+/*========================================================================*/
+
+typedef void (*Scheme_Signal_Handler_Proc)(int);
+
 /*========================================================================*/
 /*                              addrinfo                                  */
 /*========================================================================*/
@@ -2138,34 +2130,11 @@ extern Scheme_Extension_Table *scheme_extension_table;
 /*                           file descriptors                             */
 /*========================================================================*/
 
-#if defined(DETECT_WIN32_CONSOLE_STDIN) || defined(WINDOWS_PROCESSES)
-# ifndef NO_STDIO_THREADS
-#  define USE_FAR_MZ_FDCALLS
-# endif
-#endif
-#ifdef USE_DYNAMIC_FDSET_SIZE
-# define USE_FAR_MZ_FDCALLS
-#endif
-#ifdef USE_BEOS_PORT_THREADS
-# define USE_FAR_MZ_FDCALLS
-#endif
-#ifdef HAVE_POLL_SYSCALL
-# define USE_FAR_MZ_FDCALLS
-#endif
-
-#ifdef USE_FAR_MZ_FDCALLS
 # define MZ_GET_FDSET(p, n) scheme_get_fdset(p, n)
 # define MZ_FD_ZERO(p) scheme_fdzero(p)
 # define MZ_FD_SET(n, p) scheme_fdset(p, n)
 # define MZ_FD_CLR(n, p) scheme_fdclr(p, n)
 # define MZ_FD_ISSET(n, p) scheme_fdisset(p, n)
-#else
-# define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) XFORM_OK_PLUS n))
-# define MZ_FD_ZERO(p) XFORM_HIDE_EXPR(FD_ZERO((fd_set *)(p)))
-# define MZ_FD_SET(n, p) FD_SET(n, (fd_set *)(p))
-# define MZ_FD_CLR(n, p) FD_CLR(n, (fd_set *)(p))
-# define MZ_FD_ISSET(n, p) FD_ISSET(n, (fd_set *)(p))
-#endif
 
 /* For scheme_fd_to_semaphore(): */
 #define MZFD_CREATE_READ  1
@@ -2173,9 +2142,6 @@ extern Scheme_Extension_Table *scheme_extension_table;
 #define MZFD_CHECK_READ   3
 #define MZFD_CHECK_WRITE  4
 #define MZFD_REMOVE       5
-#define MZFD_CREATE_VNODE 6
-#define MZFD_CHECK_VNODE  7
-#define MZFD_REMOVE_VNODE 8
 
 /*========================================================================*/
 
