@@ -19,61 +19,6 @@
 /*   (on Windows, this is done by a manual file which is in src/worksp) */
 #include "mzconfig.h"
 
-  /*************** OSKit with Linux/gcc *****************/
-
-#if defined(OSKIT)
-
-# define SYSTEM_TYPE_NAME "oskit"
-
-# define DO_STACK_CHECK
-# ifndef OSKIT_TEST
-#  define OSKIT_FIXED_STACK_BOUNDS
-# else
-#  define ASSUME_FIXED_STACK_SIZE
-#  define FIXED_STACK_SIZE 65500
-# endif
-# define STACK_SAFETY_MARGIN 10000
-
-# define UNIX_FILE_SYSTEM
-# define NO_UNIX_USERS
-
-# define DIR_FUNCTION
-# define DIRENT_NO_NAMLEN
-# define GETENV_FUNCTION
-
-# ifndef OSKIT_TEST
-#  define HAS_FREEBSD_IOB
-# else
-#  define HAS_LINUX_IOB
-# endif
-# define USE_OSKIT_CONSOLE
-# define FILES_HAVE_FDS
-# define USE_FD_PORTS
-# define FIXED_FD_LIMIT 256
-# define USE_UNIX_SOCKETS_TCP
-# define USE_FCNTL_O_NONBLOCK
-
-# ifndef OSKIT_TEST
-#  define INCLUDE_OSKIT_SOCKET
-#  define MZ_PF_INET OSKIT_PF_INET
-#  define PROTOENT_IS_INT IPPROTO_TCP
-# endif
-
-# define NO_USLEEP
-# define NO_SLEEP
-# define USER_TIME_IS_CLOCK
-
-# define SIGSET_IS_SIGNAL
-
-# define UNISTD_INCLUDE
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
-# define FLAGS_ALREADY_SET
-
-#else  /* >>>>>>> close after all platforms <<<<<<<<<< */
-
   /************** SunOS/Solaris with gcc ****************/
 
 #if defined(sun)
@@ -92,23 +37,9 @@
 #  else
 #   define SCHEME_PLATFORM_LIBRARY_SUBPATH "sparc-solaris"
 #  endif
-#  define DIRENT_NO_NAMLEN
-#  define NO_USLEEP
-#  define USE_ULIMIT
-#  define USE_FCNTL_O_NONBLOCK
-#  define SOME_FDS_ARE_NOT_SELECTABLE
-#  define NEED_RESET_STDOUT_BLOCKING
-#  undef USE_FLOCK_FOR_FILE_LOCKS
-#  define USE_FCNTL_AND_FORK_FOR_FILE_LOCKS
-#  define USE_TIMEZONE_AND_ALTZONE_VAR
-#  define USE_TZNAME_VAR
-#  define USE_NULL_TO_DISCONNECT_UDP
 # else
 /* SunOS4 */
 # define SCHEME_PLATFORM_LIBRARY_SUBPATH "sparc-sunos4"
-# define SIGSET_IS_SIGNAL
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
 # define NO_STRERROR_AVAILABLE
 # define USE_ON_EXIT_FOR_ATEXIT
 # endif
@@ -117,10 +48,6 @@
 
 # define FMOD_CAN_RETURN_POS_ZERO
 
-# ifdef _POSIX_PTHREAD_SEMANTICS
-#  define SUBPROCESS_USE_FORK1
-# endif
-
 # ifdef __i386__
 #  define MZ_USE_JIT_I386
 #  define MZ_JIT_USE_MPROTECT
@@ -142,17 +69,9 @@
 # define SCHEME_PLATFORM_LIBRARY_SUBPATH "rs6k-aix"
 
 # include "uconfig.h"
-# undef USE_FCHDIR
 
 # define UNIX_LIMIT_STACK 33554944
 
-# define SELECT_INCLUDE
-
-# define USE_FCNTL_O_NONBLOCK
-
-# define USE_TIMEZONE_VAR_W_DLS
-# define USE_TZNAME_VAR
-
 # define FLAGS_ALREADY_SET
 
 #endif
@@ -173,9 +92,16 @@
 #  define MZ_TRY_EXTFLONUMS
 #  define ASM_DBLPREC_CONTROL_87
 # endif
-# if defined(__powerpc__)
+# if defined(__powerpc__) && !defined(__powerpc64__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-"SPLS_LINUX
 # endif
+# if defined(__powerpc64__)
+#  if defined(__LITTLE_ENDIAN__)
+#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc64le-"SPLS_LINUX
+#  else
+#   define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc64-"SPLS_LINUX
+#  endif
+# endif
 # if defined(__mc68000__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "m68k-"SPLS_LINUX
 # endif
@@ -207,33 +133,14 @@
 # endif
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
 # ifndef __ELF__
 #  undef UNIX_DYNAMIC_LOAD
 # endif
 
-# define DIRENT_NO_NAMLEN
-
-# define HAS_LINUX_IOB
-
 # define USE_IEEE_FP_PREDS
 # define USE_EXPLICT_FP_FORM_CHECK
 
-# define SIGSET_IS_SIGNAL
-# define SIGSET_NEEDS_REINSTALL
-
 # define LINUX_FIND_STACK_BASE
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define USE_TIMEZONE_VAR_W_DLS
-# define USE_TZNAME_VAR
-
-# define MZ_TCP_LISTEN_IPV6_ONLY_SOCKOPT
-
-# ifdef __ANDROID__
-#  define USE_FCNTL_O_NONBLOCK
-#  define PROTOENT_IS_INT IPPROTO_TCP
-# endif
 
 # define FLAGS_ALREADY_SET
 
@@ -247,14 +154,11 @@
 # define MZ_JIT_USE_MPROTECT
 # define MZ_USE_DWARF_LIBUNWIND
 #endif
-#if defined(__powerpc__)
+#if defined(__powerpc__) && !defined(__powerpc64__)
 # define MZ_USE_JIT_PPC
 #endif
 # if defined(__arm__)
 # define MZ_USE_JIT_ARM
-# ifndef __ANDROID__
-#  define MZ_USE_DWARF_LIBUNWIND
-# endif
 #endif
 
 #endif
@@ -275,22 +179,12 @@
 
 # include "uconfig.h"
 
-# undef HAS_STANDARD_IOB
-# define HAS_BSD_IOB
-
 #ifndef __ELF__
 # define UNDERSCORE_DYNLOAD_SYMBOL_PREFIX
 #endif
 
 # define USE_IEEE_FP_PREDS
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
 #if defined(__alpha__)
 # define USE_DIVIDE_MAKE_INFINITY
 #endif
@@ -340,8 +234,6 @@
 # endif
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
-# define HAS_BSD_IOB
 # undef UNIX_FIND_STACK_BOUNDS
 # define PTHREAD_STACKSEG_FIND_STACK_BOUNDS
 
@@ -355,13 +247,6 @@
 
 # define USE_IEEE_FP_PREDS
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
 # define USE_MAP_ANON
 
 # if defined(__x86_64__)
@@ -404,8 +289,6 @@
 # endif
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
-# define HAS_BSD_IOB
 # undef UNIX_FIND_STACK_BOUNDS
 # define PTHREAD_STACKSEG_FIND_STACK_BOUNDS
 
@@ -419,13 +302,6 @@
 
 # define USE_IEEE_FP_PREDS
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
 # define USE_MAP_ANON
 
 # if defined(__x86_64__)
@@ -468,6 +344,9 @@
 #  define FLUSH_SPARC_REGISTER_WINDOWS
 # elif defined(__arm__)
 #  define SCHEME_PLATFORM_LIBRARY_SUBPATH "arm-freebsd"
+# elif defined(__powerpc__)
+#  define SCHEME_PLATFORM_LIBRARY_SUBPATH "ppc-freebsd"
+#  define MZ_USE_JIT_PPC
 # else
 #  error Unported platform.
 # endif
@@ -478,9 +357,6 @@
 # define FIXED_STACK_SIZE 1048576
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
-
-# define HAS_BSD_IOB
 
 # ifdef FREEBSD_VERSION_2x
 #  define UNDERSCORE_DYNLOAD_SYMBOL_PREFIX
@@ -490,14 +366,6 @@
 
 # define USE_IEEE_FP_PREDS
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-# define MAX_VALID_DATE_SECONDS_BITS 51
-
 # define MZ_JIT_USE_MPROTECT
 
 # define FLAGS_ALREADY_SET
@@ -513,19 +381,11 @@
 
 # include "uconfig.h"
 
-# define DIRENT_NO_NAMLEN
-
 # define BSTRING_INCLUDE
 
 # define DEFEAT_FP_COMP_OPTIMIZATION
 # define FMOD_CAN_RETURN_NEG_ZERO
 
-# define NO_USLEEP
-# define USE_FCNTL_O_NONBLOCK
-
-# define USE_TIMEZONE_AND_ALTZONE_VAR
-# define USE_TZNAME_VAR
-
 # define FLAGS_ALREADY_SET
 
 #endif
@@ -538,12 +398,6 @@
 
 # include "uconfig.h"
 # undef UNIX_DYNAMIC_LOAD
-# undef USE_FCHDIR
-
-# define DIRENT_NO_NAMLEN
-
-# define NO_USLEEP
-# define USE_FCNTL_O_NONBLOCK
 
 # define FLAGS_ALREADY_SET
 
@@ -575,22 +429,12 @@
 
 # include "uconfig.h"
 
-# define SOME_FDS_ARE_NOT_SELECTABLE
-# define USE_FCNTL_O_NONBLOCK
-
-# define USE_SYSCALL_GETRUSAGE
-
 # define USE_DIVIDE_MAKE_INFINITY
 # define USE_IEEE_FP_PREDS
 # define USE_EXPLICT_FP_FORM_CHECK
 # define ZERO_MINUS_ZERO_IS_POS_ZERO
 # define LOG_ZERO_ISNT_NEG_INF
 
-# define USE_ULIMIT
-
-# define USE_TIMEZONE_VAR_W_DLS
-# define USE_TZNAME_VAR
-
 # define FLAGS_ALREADY_SET
 
 #endif
@@ -603,18 +447,12 @@
 # define SCHEME_PLATFORM_LIBRARY_SUBPATH "sco-i386"
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
 #ifndef __ELF__
 # undef UNIX_DYNAMIC_LOAD
 #endif
 
-# define DIRENT_NO_NAMLEN
-
-# define HAS_SCO_IOB
-
 # define USE_SCO_IEEE_FP_PREDS
 # define USE_EXPLICT_FP_FORM_CHECK
-# define USE_FCNTL_O_NONBLOCK
 
 # define REGISTER_POOR_MACHINE
 
@@ -637,20 +475,6 @@
 
 # define SYSTEM_TYPE_NAME "windows"
 # define DOS_FILE_SYSTEM
-# if defined(_MSC_VER) || defined(__MINGW32__)
-#  define NO_READDIR
-#  define USE_FINDFIRST
-#  define MKDIR_NO_MODE_FLAG
-# endif
-# if defined(__BORLANDC__)
-#  define DIRENT_NO_NAMLEN
-#  define MKDIR_NO_MODE_FLAG
-# endif
-
-# define USE_WIN32_TIME
-# define WINDOWS_GET_PROCESS_TIMES
-# define GETENV_FUNCTION
-# define DIR_FUNCTION
 
 # define USE_WINDOWS_EVENT_LOG
 # define INIT_SYSLOG_LEVEL SCHEME_LOG_ERROR
@@ -672,6 +496,7 @@
          /* With VC 7, ATAN2_DOESNT... wasn't needed, and
             POW_HANDLES_INF_CORRECTLY worked, too. */
 # define SIN_COS_NEED_DEOPTIMIZE
+# define AVOID_INT_TO_FLOAT_TRUNCATION
 #endif
 #ifdef __BORLANDC__
 # define NAN_EQUALS_ANYTHING
@@ -690,20 +515,14 @@
 # define IO_INCLUDE
 # define DONT_IGNORE_PIPE_SIGNAL
 
-# define PROCESS_FUNCTION
-# define NO_SLEEP
 # define WINDOWS_PROCESSES
 # define WINDOWS_FILE_HANDLES
 # ifndef MZ_USE_DETERMINSTIC_FUEL
 #  define USE_WIN32_THREAD_TIMER
 # endif
 
-# define SIGSET_IS_SIGNAL
-# define SIGSET_NEEDS_REINSTALL
 # define NO_SIGHUP_HANDLER
 
-#define USE_WINSOCK_TCP
-
 #define PRINTF_INTPTR_SIZE_PREFIX "I"
 
 /* MS Visual C++ likes underscore prefixes */
@@ -768,29 +587,11 @@
 
 # undef USE_ITIMER
 
-# undef HAS_STANDARD_IOB
-# define HAS_CYGWIN_IOB
-# define MZ_BINARY O_BINARY
-
-# define DIRENT_NO_NAMLEN
-# define UNISTD_INCLUDE
-
-# define SIGCHILD_DOESNT_INTERRUPT_SELECT
-# define SIGSET_IS_SIGNAL
-# define SIGSET_NEEDS_REINSTALL
-
-# define CANT_SET_SOCKET_BUFSIZE
-# define NO_NEED_FOR_BEGINTHREAD
-# define USE_CREATE_PIPE
-
 # define USE_MZ_CYGWIN_SETJMP
 # define USE_MZ_SETJMP
 
 # define USE_CYGWIN_SO_SUFFIX
 
-# define USE_PLAIN_TIME
-# define USE_TOD_FOR_TIMEZONE
-
 # define USE_DIVIDE_MAKE_INFINITY
 
 # define DO_STACK_CHECK
@@ -832,9 +633,6 @@
 
 # define INIT_SYSLOG_LEVEL SCHEME_LOG_ERROR
 
-# undef HAS_STANDARD_IOB
-# define HAS_BSD_IOB
-
 #ifndef XONX
 # undef SYSTEM_TYPE_NAME
 # define SYSTEM_TYPE_NAME "macosx"
@@ -847,22 +645,11 @@
 
 # define USE_MAP_ANON
 
-# define USE_CARBON_FP_PREDS
+# define USE_IEEE_FP_PREDS
 # define TRIG_ZERO_NEEDS_SIGN_CHECK
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define MZ_TCP_LISTEN_IPV6_ONLY_SOCKOPT
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
 # define USE_UNDERSCORE_SETJMP
 
-# define UDP_DISCONNECT_EADRNOTAVAIL_OK
-
 #ifndef XONX
 # define MACOS_UNICODE_SUPPORT
 #endif
@@ -904,17 +691,8 @@
 
 # define USE_MAP_ANON
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-
 # define USE_UNDERSCORE_SETJMP
 
-# define UDP_DISCONNECT_EADRNOTAVAIL_OK
-
 # define MZ_USE_JIT_I386
 
 # define FLAGS_ALREADY_SET
@@ -931,19 +709,11 @@
 # define ASSUME_FIXED_STACK_SIZE
 
 # include "uconfig.h"
-# define SIGSET_IS_SIGNAL
-# define SIGSET_NEEDS_REINSTALL
-
-# define USE_FCNTL_O_NONBLOCK
 
 # define FIXED_STACK_SIZE 524288
 
 # define FLAGS_ALREADY_SET
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define BROKEN_READLINK_NUL_TERMINATOR
-
 #if defined(__i386__)
 # define MZ_USE_JIT_I386
 # define MZ_JIT_USE_MPROTECT
@@ -982,9 +752,6 @@
 # define FIXED_STACK_SIZE 1048576
 
 # include "uconfig.h"
-# undef HAS_STANDARD_IOB
-
-# define HAS_BSD_IOB
 
 # define USE_UNDERSCORE_SETJMP
 
@@ -993,24 +760,12 @@
 #  define POW_HANDLES_INF_CORRECTLY
 # endif
 
-# define USE_DYNAMIC_FDSET_SIZE
-
-# define SIGSET_IS_SIGNAL
-
-# define USE_TM_GMTOFF_FIELD
-# define USE_TM_ZONE_FIELD
-# define MAX_VALID_DATE_SECONDS_BITS 51
-
 # define MZ_JIT_USE_MPROTECT
 
 # define FLAGS_ALREADY_SET
 
 #endif
 
-  /***************************************************/
-
-#endif /* end not OSKit */
-
 /************** (END KNOWN ARCHITECTURE/SYSTEMS) ****************/
 
 
@@ -1020,8 +775,6 @@
 
 /* assume generic Unix: */
 #include "uconfig.h"
-# define SIGSET_IS_SIGNAL
-# define SIGSET_NEEDS_REINSTALL
 
   /*********************/
  /* Operating System  */
@@ -1035,49 +788,6 @@
      "schsys.h" into the same directory as .o files and #included
      by string.c. This string is returned by (system-library-subpath) */
 
-  /*********************/
- /* Language Features */
-/*********************/
-
- /* CLOCKS_PER_SEC relates the values returned by clock() to
-     real seconds. (The difference between two clock() calls is
-     divided by this number.) Usually, this is defined in <time.h>;
-     it defaults to 1000000 */
-
- /* USE_PLAIN_TIME uses time. */
-
- /* USE_WIN32_TIME uses the Win32 API to implement time functions. */
-
- /* CLOCK_IS_USER_TIME uses the system time for user milliseconds. */
-
- /* USER_TIME_IS_CLOCK uses the user time for system milliseconds. */
-
- /* TIME_TYPE_IS_UNSIGNED converts time_t values as unsigned. */
-
- /* MAX_VALID_DATE_SECONDS_BITS sets a maximum number of bits for
-    seconds to pass to localtime() ro gmtime(). */
-
- /* MIN_VALID_DATE_SECONDS sets a minimum vald time in seconds. */
-
- /* PROCESS_FUNCTION adds (process ...) and (system ...) functions */
-
- /* DIR_FUNCTION adds (current-directory ...) function */
-
- /* GETENV_FUNCTION adds (getenv ...) function */
-
- /* USE_TIMEZONE_VAR gets timezone offset from a timezone global.
-    USE_TOD_FOR_TIMEZONE gets timezone offset via gettimeofday.
-    USE_TIMEZONE_VAR_W_DLS is similar, but adds 1 hour when daylight
-     savings is in effect.
-    USE_TIMEZONE_AND_ALTZONE_VAR is similar, but uses altzone when
-     daylight savings is in effect.
-    USE_TM_GMTOFF_FIELD gets timezone offset from the tm_gmtoff field
-     of the tm struct. */
-
- /* USE_TZNAME_VAR gets the timezone name from a tzname global.
-    USE_TM_ZONE_FIELD gets the timezone name from a tm_zone field
-     of the tm struct. */
-
   /*******************/
  /*   Filesystem    */
 /*******************/
@@ -1091,34 +801,6 @@
     "X:\", "X:/", "\", or "/" as a root directory (for some letter X),
     and case insensitivity */
 
- /* MAC_FILE_SYSTEM indicates that filenames are as on the Macintosh,
-    with colon separators, "" as the parent directory, a volume name
-    (followed by a colon) as a root directory, and case insensitivity. */
-
- /* EXPAND_FILENAME_TILDE expands ~ in a filename with a user's home
-     directory. */
-
- /* NO_STAT_PROC means that there is no stat() function. */
-
- /* NO_MKDIR means that there is no mkdir() function. */
-
- /* BROKEN_READLINK_NUL_TERMINATOR means that readlink() may
-    report a length that includes trailing NUL terminators,
-    which should be stripped away. */
-
- /* USE_GETDISK uses getdisk() and setdisk() to implement the
-     filesystem-root-list primitive under DOS. */
-
- /* NO_READDIR means that there is no opendir() and readdir() for
-     implementing directory-list. */
-
- /* DIRENT_NO_NAMLEN specifies that dirent entries do not have a
-     d_namlen field; this is used only when NO_READDIR is not
-     specified. */
-
- /* MKDIR_NO_MODE_FLAG specifies that mkdir() takes only one argument,
-     instead of a directory name and mode flags. */
-
  /* USE_C_SYSLOG uses the C syslog library for logging. */
 
  /* USE_WINDOWS_EVENT_LOG uses the Windows event log API for logging. */
@@ -1126,6 +808,10 @@
  /* INIT_SYSLOG_LEVEL sets the initial level for filtering messages
     sent to syslog. It default to 0 (i.e., no events). */
 
+ /* USE_TRANSITIONAL_64_FILE_OPS uses lseek64, stat64, etc. for file
+    operations involving sizes (that can require 64-bit
+    arithmetic). */
+
   /***********************/
  /*       Ports         */
 /***********************/
@@ -1134,101 +820,16 @@
    None of these flags are required, but char-ready? may return
    spurious #ts if they are set up incorrectly. */
 
- /* HAS_STANDARD_IOB, HAS_GNU_IOB, HAS_CYGWIN_IOB, HAS_LINUX_IOB,
-    HAS_BSD_IOB, and HAS_SCO_IOB are mutually exclusive; they describe
-    how to read the FILE* structure to determine if there are
-    available cached characters. */
-
- /* USE_FD_PORTS uses Unix-style open(), etc., for file ports. */
-
- /* FILES_HAVE_FDS means that a FILE* is always associated with a
-    file desciptor, which can be select-ed to see if there are
-    pending bytes. Don't use this unless one of the HAS_<X>_IOB
-    flags is used. */
-
- /* USE_FLOCK_FOR_FILE_LOCKS means that flock() is available and works
-    for file locking. */
-
- /* USE_FCNTL_AND_FORK_FOR_FILE_LOCKS means that fnctl() and fork()
-    should be used to implement file locking. */
-
- /* CLOSE_ALL_FDS_AFTER_FORK means that all fds except 0, 1, and 2
-    should be closed after performing a fork() for `process'
-    and `system' calls. */
-
- /* SUBPROCESS_USE_FORK1 uses fork1() instead of fork(). */
-
- /* USE_UNIX_SOCKETS_TCP means that the tcp- procedures can be implemented
-    with the standard Unix socket functions. */
-
- /* USE_WINSOCK_TCP means that the tcp- procedures can be implemented
-    with the Winsock toolbox. */
-
- /* USE_MAC_TCP means that the tcp- procedures can be implemented
-    with the Mac TCP toolbox. */
-
  /* WINDOWS_FILE_HANDLES implements file access through the Windows
     API (CreateFile(), ReadFile(), etc.) */
 
- /* USE_FCNTL_O_NONBLOCK uses O_NONBLOCK instead of FNDELAY for
-    fcntl on Unix TCP sockets. (Posix systems need this flag). */
-
- /* SOME_FDS_ARE_NOT_SELECTABLE indicates that select() doesn't work
-    for reading on all kinds of file descriptors. Such FDs must never
-    be able to go from no-char-ready to char-ready while Racket is
-    sleeping. */
-
- /* NEED_RESET_STDOUT_BLOCKING enures that file descriptors 1 and 2
-    are reset to blocking mode before exiting. */
-
- /* USE_TRANSITIONAL_64_FILE_OPS uses fseeko64, lseek64, stat64,
-    etc. for file operations involving sizes (that can require
-    64-bit arithmetic). */
-
- /* USE_ULIMIT uses ulimit instead of getdtablesize (Unix). */
-
- /* USE_DYNAMIC_FDSET_SIZE allocates fd_set records based on the
-    current fd limit instead of relying on the compile-time size
-    of fd_set. */
-
- /* UNIX_LIMIT_FDSET_SIZE insures that the fd limit at start-up is
-    no greater than FD_SETSIZE */
-
- /* CANT_SET_SOCKET_BUFSIZE turns off setting the buffer size for
-    Unix TCP sockets. */
-
- /* USE_NULL_TO_DISCONNECT_UDP calls connect() with NULL instead of
-    an AF_UNSPEC address to disconnect a UDP socket. */
-
- /* UDP_DISCONNECT_EADRNOTAVAIL_OK means that a disconnecting call
-    to connect() might return EADDRNOTAVAIL instead of
-    EAFNOSUPPORT. */
-
- /* MZ_BINARY is combined with other flags in all calls to open();
-    it can be defined to O_BINARY in Cygwin, for example. */
-
- /* MZ_TCP_LISTEN_IPV6_ONLY_SOCKOPT uses IPV6_V6ONLY for IPv6
-    listeners when the same listener has an IPv4 address, which means
-    that the IpV6 listener accepts only IPv6 connections. This is used
-    with Linux, for example, because a port cannot have both an IPv4
-    and IPv6 listener if the IPv6 one doesn't use IPV6_V6ONLY. (The
-    two listeners might be for different interfaces, in which case
-    IPV6_V6ONLY is not necessary, but we must err on the side of being
-    too restrictive. If IPV6_V6ONLY is not #defined or if setting the
-    option doesn't work, then the IPv6 addresses are silently ignored
-    when creating the listener (but only where there is at least once
-    IPv4 address). */
-
   /***********************/
  /* Threads & Signals  */
 /***********************/
 
 /* These are flags about the implementation of system, process, etc. */
 
- /* UNIX_PROCESSES implements the process functions for Unix; uses
-    sigset() to install the signal handler. */
-
- /* WINDOWS_PROCESSES implements the process functions for Windows. */
+ /* WINDOWS_PROCESSES uses Windows threads for various tasks. */
 
  /* USE_ITIMER uses setitimer() to implement thread pre-emption (for
     Racket-implemented threads). Define MZ_THREAD_QUANTUM_USEC to
@@ -1240,27 +841,12 @@
  /* USE_PTHREAD_THREAD_TIMER uses a background pthread to implement
     tread pre-emption. */
 
- /* SIGSET_IS_SIGNAL uses signal() in place of sigset() for Unix. This
-    flag is often paired with SIGSET_NEEDS_REINSTALL for traditional
-    Unix systems. */
-
- /* SIGSET_NEEDS_REINSTALL reinstalls a signal handler when it
-    is called to handle a signal. The expected semantics of sigset()
-    (when this flags is not defined) is that a signal handler is NOT
-    reset to SIG_DFL after a handler is called to handle a signal. */
-
  /* DONT_IGNORE_FPE_SIGNAL stops Racket from ignoring floating-point
     exception signals. */
 
  /* DONT_IGNORE_PIPE_SIGNAL stops Racket from ignoring SIGPIPE
     signals. */
 
- /* USE_CREATE_PIPE uses CreatePipe() instead of _pipe() for Windows. */
-
- /* SIGCHILD_DOESNT_INTERRUPT_SELECT indicates that the SIGCHILD
-    signal, sent when a child OS process dies, does not interrupt
-    select(). This flag is needed for Cygwin B20. */
-
  /* USE_UNDERSCORE_SETJMP uses _setjmp() instead of setjmp() to avoid
     sigmal-mask work. */
 
@@ -1274,6 +860,11 @@
  /* FLOATING_POINT_IS_NOT_IEEE disables inexact->exact conversion via
     parsing of IEEE-format bits. */
 
+ /* AVOID_INT_TO_FLOAT_TRUNCATION indicates that conversion from an
+    integer to a floating point type does not round-to-nearest when
+    precision is lost, even when the FP rounding mode is
+    round-to-nearest */
+
  /* USE_SINGLE_FLOATS turns on support for single-precision
     floating point numbers. Otherwise, floating point numbers
     are always represented in double-precision. */
@@ -1405,7 +996,7 @@
  /* SIN_COS_NEED_DEOPTIMIZE disables optimization for calls to sin()
     and cos() (for MSVC) */
 
-/******************************/
+  /****************************/
  /* Byte Order and long long */
 /****************************/
 
@@ -1452,9 +1043,6 @@
      be necessary to avoid GC-setup traversal over too much memory
      (with GC flag HEURISTIC2?). */
 
- /* OSKIT_FIXED_STACK_BOUNDS uses base_stack_start to get the hottest
-     stack location */
-
   /***********************/
  /*   Dynamic Loading   */
 /***********************/
@@ -1502,39 +1090,13 @@
  /* MZ_JIT_USE_WINDOWS_VIRTUAL_ALLOC uses VirtualAlloc to make
     code pages executable. */
 
-  /***********************/
- /*     Heap Images     */
-/***********************/
-
- /* UNIX_IMAGE_DUMPS turns on image save and restore for Unix systems.
-     This will only work if the final application is statically linked.
-     (As an exception, the dynamic-linking library itself can be
-     dynamically linked. This works because loading an extension in
-     Racket automatically turns off image saving.) */
-
   /*****************************/
- /*   Macintosh Standalone    */
+ /*   MacOS                   */
 /*****************************/
 
- /* MACINTOSH_EVENTS enables specific support for Mac Classic */
-
- /* MAC_MZ_GUI_ENABLED activates the windowed I/O code (use under classic)
-     (should maybe use MACINTOSH_SIOUX instead?) */
-
- /* MAC_CLASSIC_PROCESS_CONTROL swaps the UNIX process commands for the
-    mac family (use under classic) */
-
- /* OS_X enables specific support for Mac OS X, e.g. the location of the
+ /* OS_X enables specific support for Mac OS, e.g. the location of the
     prefs directory */
 
- /* MACINTOSH_GIVE_TIME lets background processes run when checking for
-     a user break */
-
- /* MACINTOSH_SIOUX interfaces with Metrowerks's SIOUX library */
-
- /* MACINTOSH_SET_STACK sets the stack to be 1/4 of the heap; this should
-     be used for 68k machines, where the stack is not user-configurable */
-
   /***********************/
  /*    Miscellaneous    */
 /***********************/
@@ -1574,51 +1136,11 @@
 
  /* NO_SIGHUP_HANDLER turns off handling of SIGHUP in main.c */
 
- /* DIR_INCLUDE if there's a <dir.h> file (mainly for Windows). */
-
- /* DIRECT_INCLUDE if there's a <direct.h> file (mainly for Windows). */
-
- /* IO_INCLUDE if there's a <io.h> file (mainly for Windows). */
-
- /* UNISTD_INCLUDE if there's a <unistd.h> file (mainly for Unix). */
-
- /* SELECT_INCLUDE if there's a <sys/select.h> file (mainly for Unix)
-     to be used with FILES_HAVE_FDS. */
-
- /* BSTRING_INCLUDE if there's a <bstring.h> file (mainly for Unix)
-     to be used with FILES_HAVE_FDS. */
-
- /* NO_SYS_INCLUDE_SUBDIR if include files should all be <XXX.h>; no
-     includes of the form <sys/XXX.h>. Mainly used for <sys/stat.h>
-     for MacOS. */
-
- /* USE_FCHDIR uses fchdir() to improve thread context switches when
-    a small number of threads are active. */
-
- /* USE_GETRUSAGE uses getrusage() to for timing info; otherwise clock()
-    is used. */
-
- /* USE_SYSCALL_GETRUSAGE uses syscall() to implement getrusage() for
-    timing info. Used with USE_GETRUSAGE. */
-
- /* NO_SLEEP means that there is no sleep() function. Used only in
-    standalone Racket. */
-
- /* NO_USLEEP means that there is no usleep() function. Used only in
-    standalone Racket. Used only if NO_SLEEP is undefined. */
-
  /* NO_STRERROR_AVAILABLE means that strerror() is not available. */
 
  /* USE_ON_EXIT_FOR_ATEXIT means that a SunOS4-style on_exit()
     is available instead of atexit(). */
 
- /* NO_NEED_FOR_BEGINTHREAD indicates that the C library used for
-    Windows is always thread-ready and there's no need use the
-    _beginthreadex() function instead of CreateThread(). */
-
- /* WIN32S_HACK uses a special hack to implement threads under Win32s
-    with some compilers. Obsolete. */
-
  /* FFI_CALLBACK_NEED_INT_CLEAR indiates thet libffi callback results
     that are smaller than an `int' should clear `int'-sized space
     in the result area. */
