@@ -20,8 +20,8 @@ extern "C" {
 
 SGC_EXTERN void GC_add_roots(void *start, void *end);
 
-SGC_EXTERN void (*GC_start_collect_callback)(void);
-SGC_EXTERN void (*GC_end_collect_callback)(void);
+SGC_EXTERN void (*GC_collect_start_callback)(void);
+SGC_EXTERN void (*GC_collect_end_callback)(void);
 SGC_EXTERN void (*GC_custom_finalize)(void);
 SGC_EXTERN void (*GC_out_of_memory)(void);
 
@@ -35,7 +35,8 @@ SGC_EXTERN void *GC_base(void *);
 
 SGC_EXTERN void GC_dump(void);
 
-SGC_EXTERN long GC_get_memory_use();
+SGC_EXTERN size_t GC_get_memory_use();
+SGC_EXTERN size_t GC_get_total_bytes();
 
 SGC_EXTERN void GC_end_stubborn_change(void *);
 
