@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2016 PLT Design Inc.
+  Copyright (c) 2004-2017 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -35,12 +35,15 @@ READ_ONLY Scheme_Object scheme_true[1];
 READ_ONLY Scheme_Object scheme_false[1];
 
 READ_ONLY Scheme_Object *scheme_not_proc;
+READ_ONLY Scheme_Object *scheme_true_object_p_proc;
+READ_ONLY Scheme_Object *scheme_boolean_p_proc;
 READ_ONLY Scheme_Object *scheme_eq_proc;
 READ_ONLY Scheme_Object *scheme_eqv_proc;
 READ_ONLY Scheme_Object *scheme_equal_proc;
 
 /* locals */
 static Scheme_Object *not_prim (int argc, Scheme_Object *argv[]);
+static Scheme_Object *true_object_p_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *boolean_p_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *eq_prim (int argc, Scheme_Object *argv[]);
 static Scheme_Object *eqv_prim (int argc, Scheme_Object *argv[]);
@@ -84,6 +87,8 @@ void scheme_init_bool (Scheme_Env *env)
   Scheme_Object *p;
 
   REGISTER_SO(scheme_not_proc);
+  REGISTER_SO(scheme_true_object_p_proc);
+  REGISTER_SO(scheme_boolean_p_proc);
   REGISTER_SO(scheme_eq_proc);
   REGISTER_SO(scheme_eqv_proc);
   REGISTER_SO(scheme_equal_proc);
@@ -94,9 +99,16 @@ void scheme_init_bool (Scheme_Env *env)
                                                             | SCHEME_PRIM_IS_OMITABLE);
   scheme_add_global_constant("not", p, env);
 
+  p = scheme_make_folding_prim(true_object_p_prim, "true-object?", 1, 1, 1);
+  SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_UNARY_INLINED
+                                                            | SCHEME_PRIM_IS_OMITABLE);
+  scheme_true_object_p_proc = p;
+  scheme_add_global_constant("true-object?", p, env);
+
   p = scheme_make_folding_prim(boolean_p_prim, "boolean?", 1, 1, 1);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_UNARY_INLINED
                                                             | SCHEME_PRIM_IS_OMITABLE);
+  scheme_boolean_p_proc = p;
   scheme_add_global_constant("boolean?", p, env);
 
   p = scheme_make_folding_prim(eq_prim, "eq?", 2, 2, 1);
@@ -111,7 +123,7 @@ void scheme_init_bool (Scheme_Env *env)
   scheme_eqv_proc = p;
   scheme_add_global_constant("eqv?", scheme_eqv_proc, env);
   
-  p = scheme_make_prim_w_arity(equal_prim, "equal?", 2, 2);
+  p = scheme_make_noncm_prim(equal_prim, "equal?", 2, 2);
   SCHEME_PRIM_PROC_FLAGS(p) |= scheme_intern_prim_opt_flags(SCHEME_PRIM_IS_BINARY_INLINED);
   scheme_equal_proc = p;
   scheme_add_global_constant("equal?", scheme_equal_proc, env);
@@ -147,6 +159,12 @@ not_prim (int argc, Scheme_Object *argv[])
   return (SAME_OBJ(argv[0], scheme_false) ? scheme_true : scheme_false);
 }
 
+static Scheme_Object *
+true_object_p_prim (int argc, Scheme_Object *argv[])
+{
+  return (SAME_OBJ(argv[0], scheme_true) ? scheme_true : scheme_false);
+}
+
 static Scheme_Object *
 boolean_p_prim (int argc, Scheme_Object *argv[])
 {
@@ -621,7 +639,8 @@ int is_equal (Scheme_Object *obj1, Scheme_Object *obj2, Equal_Info *eql)
     }
     if (!eql->for_chaperone) {
       if (SCHEME_CHAPERONEP(obj1)) {
-        obj1 = ((Scheme_Chaperone *)obj1)->val;
+	/* OPT only use prev for unsafe-chaperone-vector, use val otherwise */
+        obj1 = ((Scheme_Chaperone *)obj1)->prev;
         goto top_after_next;
       }
       if (SCHEME_CHAPERONEP(obj2)) {
