@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2016 PLT Design Inc.
+  Copyright (c) 2004-2017 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -67,6 +67,10 @@ static void register_traversers(void);
 #define OBJ_HASH_USEFUL_BITS (16 - OBJ_HASH_USELESS_BITS)
 #define OBJ_HASH_USEFUL_MASK ((1 << OBJ_HASH_USEFUL_BITS)-1)
 
+/* A hash code has been installed if any of these bits is
+   non-zero: */
+#define HASH_CODE_PRESENT_BITS 0xFFFC
+
 #ifdef MZ_PRECISE_GC
 /* keygen race conditions below are "ok", because keygen is randomness
    used to create a hashkey. Technically, a race condition allows
@@ -88,7 +92,7 @@ uintptr_t PTR_TO_LONG(Scheme_Object *o)
 
   v = o->keyex;
 
-  if (!(v & 0xFFFC)) {
+  if (!(v & HASH_CODE_PRESENT_BITS)) {
     uintptr_t local_keygen = keygen;
     v |= (short)local_keygen;
 #ifdef OBJHEAD_HAS_HASH_BITS
@@ -128,6 +132,15 @@ uintptr_t PTR_TO_LONG(Scheme_Object *o)
 # define PTR_TO_LONG(p) ((uintptr_t)(p)>>2)
 #endif
 
+void scheme_set_distinct_eq_hash(Scheme_Object *o)
+/* Used after cloning a value to ensure that the
+   hash code is not cloned */
+{
+#ifdef MZ_PRECISE_GC
+  o->keyex = o->keyex & ~HASH_CODE_PRESENT_BITS;
+#endif
+}
+
 void scheme_install_symbol_hash_code(Scheme_Object *sym, uintptr_t h)
 {
 #ifdef MZ_PRECISE_GC
@@ -246,8 +259,6 @@ Scheme_Hash_Table *scheme_make_hash_table(int type)
 {
   Scheme_Hash_Table *table;
 
-  DEBUG_COUNT_ALLOCATION(scheme_make_integer(scheme_hash_table_type));
-
   table = MALLOC_ONE_TAGGED(Scheme_Hash_Table);
 
   table->size = 0;
@@ -642,8 +653,6 @@ Scheme_Hash_Table *scheme_clone_hash_table(Scheme_Hash_Table *ht)
   Scheme_Hash_Table *table;
   Scheme_Object **ba;
 
-  DEBUG_COUNT_ALLOCATION(scheme_make_integer(scheme_hash_table_type));
-
   table = MALLOC_ONE_TAGGED(Scheme_Hash_Table);
   memcpy(table, ht, sizeof(Scheme_Hash_Table));
   MZ_OPT_HASH_KEY(&(table->iso)) = 0;
@@ -727,8 +736,6 @@ scheme_make_bucket_table (intptr_t size, int type)
   Scheme_Bucket_Table *table;
   size_t asize;
 
-  DEBUG_COUNT_ALLOCATION(scheme_make_integer(scheme_bucket_table_type));
-
   table = MALLOC_ONE_TAGGED(Scheme_Bucket_Table);
 
   table->size = 4;
@@ -1125,8 +1132,6 @@ Scheme_Bucket_Table *scheme_clone_bucket_table(Scheme_Bucket_Table *bt)
   Scheme_Bucket_Table *table;
   size_t asize;
 
-  DEBUG_COUNT_ALLOCATION(scheme_make_integer(scheme_bucket_table_type));
-
   table = MALLOC_ONE_TAGGED(Scheme_Bucket_Table);
   table->so.type = scheme_bucket_table_type;
   table->size = bt->size;
@@ -2502,7 +2507,7 @@ XFORM_NONGCING Scheme_Object *_mzHAMT_VAL(Scheme_Hash_Tree *ht, int pos, int pop
 XFORM_NONGCING uintptr_t mzHAMT_KEY_CODE(Scheme_Object *o)
 {
   while (1) {
-    if (HASHTR_COLLISIONP(o))
+    if (HASHTR_COLLISIONP(o) || HASHTR_SUBTREEP(o))
       o = ((Scheme_Hash_Tree *)o)->els[0];
     else {
       uintptr_t h;
@@ -2570,7 +2575,6 @@ XFORM_NONGCING static Scheme_Hash_Tree *hamt_assoc(Scheme_Hash_Tree *ht, uintptr
 static Scheme_Hash_Tree *hamt_alloc(int kind, int popcount)
 /* be sure to set `bitmap` field before a GC becomes possible */
 {
-  DEBUG_COUNT_ALLOCATION(scheme_make_integer(scheme_hash_tree_type));
   return (Scheme_Hash_Tree *)scheme_malloc_small_tagged(HASH_TREE_RECORD_SIZE(kind, popcount));
 }
 
