@@ -98,6 +98,7 @@
 # define jit_ldi_ld_fppush(rd, is) jit_fpu_ldi_ld_fppush(rd, is)
 # define jit_ldr_d(rd, rs) jit_fpu_ldr_d(rd, rs)
 # define jit_ldr_d_fppush(rd, rs) jit_fpu_ldr_d_fppush(rd, rs)
+# define jit_ldr_f_fppush(rd, rs) jit_fpu_ldr_f_fppush(rd, rs)
 # define jit_ldr_ld(rd, rs) jit_fpu_ldr_ld(rd, rs)
 # define jit_ldr_ld_fppush(rd, rs) jit_fpu_ldr_ld_fppush(rd, rs)
 # define jit_ldxi_d(rd, rs, is) jit_fpu_ldxi_d(rd, rs, is)
@@ -113,16 +114,20 @@
 # define jit_extr_i_ld_fppush(rd, rs) jit_fpu_extr_i_ld_fppush(rd, rs)
 # define jit_extr_l_d_fppush(rd, rs) jit_fpu_extr_l_d_fppush(rd, rs)
 # define jit_extr_l_ld_fppush(rd, rs) jit_fpu_extr_l_ld_fppush(rd, rs)
+# define jit_extr_d_f(rd, rs) jit_fpu_extr_d_f(rd, rs)
+# define jit_extr_f_d(rd, rs) jit_fpu_extr_f_d(rd, rs)
 # define jit_stxi_f(id, rd, rs) jit_fpu_stxi_f(id, rd, rs)
 # define jit_stxr_f(d1, d2, rs) jit_fpu_stxr_f(d1, d2, rs)
 # define jit_stxi_d(id, rd, rs) jit_fpu_stxi_d(id, rd, rs)
 # define jit_stxr_d(d1, d2, rs) jit_fpu_stxr_d(d1, d2, rs)
 # define jit_sti_d(id, rs) jit_fpu_sti_d(id, rs)
 # define jit_str_d(rd, rs) jit_fpu_str_d(rd, rs)
+# define jit_str_f(rd, rs) jit_fpu_str_f(rd, rs)
 # define jit_sti_d_fppop(id, rs) jit_fpu_sti_d_fppop(id, rs)
 # define jit_sti_ld_fppop(id, rs) jit_fpu_sti_ld_fppop(id, rs)
 # define jit_stxi_d_fppop(id, rd, rs) jit_fpu_stxi_d_fppop(id, rd, rs)
 # define jit_str_d_fppop(rd, rs) jit_fpu_str_d_fppop(rd, rs)
+# define jit_str_f_fppop(rd, rs) jit_fpu_str_f_fppop(rd, rs)
 # define jit_stxr_d_fppop(d1, d2, rs) jit_fpu_stxr_d_fppop(d1, d2, rs)
 # define jit_stxi_ld_fppop(id, rd, rs) jit_fpu_stxi_ld_fppop(id, rd, rs)
 # define jit_str_ld_fppop(rd, rs) jit_fpu_str_ld_fppop(rd, rs)
