@@ -31,6 +31,10 @@
 #include "schpriv.h"
 #include <string.h>
 #include "schgc.h"
+#include "schrktio.h"
+#ifndef WINDOWS_PROCESSES
+# include <signal.h>
+#endif
 
 #ifdef DOS_FAR_POINTERS
 # include <alloc.h>
@@ -54,6 +58,8 @@
 # include <windows.h>
 #endif
 
+THREAD_LOCAL_DECL(rktio_t *scheme_rktio);
+
 THREAD_LOCAL_DECL(uintptr_t scheme_os_thread_id);
 
 THREAD_LOCAL_DECL(static void **dgc_array);
@@ -113,12 +119,6 @@ extern MZGC_DLLIMPORT void GC_init();
 
 void scheme_set_stack_base(void *base, int no_auto_statics) XFORM_SKIP_PROC
 {
-#if defined(MZ_PLACES_WAITPID)
-  /* Early, to maximize the chance that no threads have been
-     created that might later receive SIGCHLD */
-  scheme_places_block_child_signal();
-#endif
-
 #ifdef MZ_PRECISE_GC
   GC_init_type_tags(_scheme_last_type_, 
                     scheme_pair_type, scheme_mutable_pair_type, scheme_weak_box_type, 
@@ -321,6 +321,7 @@ int scheme_main_stack_setup(int no_auto_statics, Scheme_Nested_Main _main, void
 {
   scheme_setup_thread_local_key_if_needed();
   scheme_init_os_thread();
+  scheme_rktio = rktio_init();
 #ifdef MZ_USE_MZRT
   scheme_init_glib_log_queue();
 #endif
@@ -447,6 +448,17 @@ int scheme_is_place_main_os_thread() XFORM_SKIP_PROC
   return 1;
 }
 
+void scheme_set_signal_handler(int sig_id, Scheme_Signal_Handler_Proc proc) XFORM_SKIP_PROC
+{
+#ifndef WINDOWS_PROCESSES
+  struct sigaction sa;
+  sigemptyset(&sa.sa_mask);
+  sa.sa_flags = 0;
+  sa.sa_handler = (proc ? proc : SIG_IGN);
+  sigaction(sig_id, &sa, NULL);
+#endif
+}
+
 /************************************************************************/
 /*                           memory utils                               */
 /************************************************************************/
@@ -542,7 +554,7 @@ scheme_calloc (size_t num, size_t size)
   memset(space, 0, (num*size));
 #endif
 
-  return (space);
+  return space;
 }
 
 char *
@@ -553,7 +565,7 @@ scheme_strdup(const char *str)
 
   len = strlen(str) + 1;
   naya = (char *)scheme_malloc_atomic (len * sizeof (char));
-  memcpy (naya, str, len);
+  memcpy(naya, str, len);
 
   return naya;
 }
@@ -566,7 +578,22 @@ scheme_strdup_eternal(const char *str)
 
   len = strlen(str) + 1;
   naya = (char *)scheme_malloc_eternal(len * sizeof (char));
-  memcpy (naya, str, len);
+  memcpy(naya, str, len);
+
+  return naya;
+}
+
+char *
+scheme_strdup_and_free(const char *str)
+{
+  char *naya;
+  intptr_t len;
+
+  len = strlen(str) + 1;
+  naya = (char *)scheme_malloc_atomic (len * sizeof (char));
+  memcpy(naya, str, len);
+
+  free((void *)str);
 
   return naya;
 }
