@@ -36,19 +36,22 @@ enum {
   CPT_DELAY_REF,
   CPT_PREFAB,
   CPT_LET_ONE_UNUSED,
+  CPT_SCOPE,
+  CPT_ROOT_SCOPE,
+  CPT_SHARED,
   _CPT_COUNT_
 };
 
-#define CPT_SMALL_NUMBER_START 36
-#define CPT_SMALL_NUMBER_END   60
+#define CPT_SMALL_NUMBER_START 39
+#define CPT_SMALL_NUMBER_END   62
 
-#define CPT_SMALL_SYMBOL_START 60
+#define CPT_SMALL_SYMBOL_START 62
 #define CPT_SMALL_SYMBOL_END   80
 
 #define CPT_SMALL_MARSHALLED_START 80
 #define CPT_SMALL_MARSHALLED_END 92
 
-#define _SMALL_LIST_MAX_ 65
+#define _SMALL_LIST_MAX_ 50
 
 #define CPT_SMALL_PROPER_LIST_START 92
 #define CPT_SMALL_PROPER_LIST_END  (CPT_SMALL_PROPER_LIST_START + _SMALL_LIST_MAX_) 
