@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2015 PLT Design Inc.
+  Copyright (c) 2004-2017 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
   All rights reserved.
 
@@ -123,6 +123,7 @@ int (*scheme_flush_managed)(Scheme_Plumber *p, int catch_errors);
 Scheme_Object *(*scheme_add_flush)(Scheme_Plumber *p, Scheme_Object *proc_or_port, int weak_flush);
 void (*scheme_remove_flush)(Scheme_Object *h);
 void (*scheme_add_atexit_closer)(Scheme_Exit_Closer_Func f);
+int (*scheme_atexit)(void (*func)(void));
 void (*scheme_add_evt)(Scheme_Type type,
 				   Scheme_Ready_Fun ready,
 				   Scheme_Needs_Wakeup_Fun wakeup,
@@ -147,7 +148,7 @@ void (*scheme_push_break_enable)(Scheme_Cont_Frame_Data *cframe, int on, int pre
 void (*scheme_pop_break_enable)(Scheme_Cont_Frame_Data *cframe, int post_check);
 Scheme_Object *(*scheme_abort_continuation_no_dws)(Scheme_Object *pt, Scheme_Object *v);
 Scheme_Object *(*scheme_call_with_composable_no_dws)(Scheme_Object *proc, Scheme_Object *pt);
-Scheme_On_Atomic_Timeout_Proc (*scheme_set_on_atomic_timeout)(Scheme_On_Atomic_Timeout_Proc p);
+Scheme_On_Atomic_Timeout_Proc (*scheme_set_on_atomic_timeout)(Scheme_On_Atomic_Timeout_Proc p, void *data);
 /*========================================================================*/
 /*                              error handling                            */
 /*========================================================================*/
@@ -170,6 +171,7 @@ void (*scheme_log_name_pfx_message)(Scheme_Logger *logger, int level, Scheme_Obj
 void (*scheme_log_abort)(char *buffer);
 void (*scheme_log_warning)(char *buffer);
 void (*scheme_glib_log_message)(const char *log_domain, int log_level, const char *message, void *user_data);
+void *(*scheme_glib_log_message_test)(char *str);
 void (*scheme_out_of_memory_abort)();
 void (*scheme_wrong_count)(const char *name, int minc, int maxc,
 				  int argc, Scheme_Object **argv);
@@ -363,6 +365,7 @@ void (*scheme_gc_ptr_ok)(void *p);
 void (*scheme_collect_garbage)(void);
 void (*scheme_collect_garbage_minor)(void);
 void (*scheme_enable_garbage_collection)(int on);
+void (*scheme_incremental_garbage_collection)(int on);
 #ifdef MZ_PRECISE_GC
 # ifndef USE_THREAD_LOCAL
 void **GC_variable_stack;
@@ -378,6 +381,7 @@ void **(*scheme_malloc_immobile_box)(void *p);
 void (*scheme_free_immobile_box)(void **b);
 Scheme_Object *(*scheme_add_gc_callback)(Scheme_Object *pre, Scheme_Object *post);
 void (*scheme_remove_gc_callback)(Scheme_Object *key);
+void (*scheme_register_type_gc_shape)(Scheme_Type type, intptr_t *shape_str);
 /*========================================================================*/
 /*                             hash tables                                */
 /*========================================================================*/
@@ -389,6 +393,8 @@ Scheme_Bucket *(*scheme_bucket_from_table)(Scheme_Bucket_Table *table, const cha
 int (*scheme_bucket_table_equal)(Scheme_Bucket_Table *t1, Scheme_Bucket_Table *t2);
 Scheme_Bucket_Table *(*scheme_clone_bucket_table)(Scheme_Bucket_Table *bt);
 void (*scheme_clear_bucket_table)(Scheme_Bucket_Table *bt);
+int (*scheme_bucket_table_index)(Scheme_Bucket_Table *hash, mzlonglong pos, Scheme_Object **_key, Scheme_Object **_val);
+Scheme_Object *(*scheme_bucket_table_next)(Scheme_Bucket_Table *hash, mzlonglong start);
 Scheme_Hash_Table *(*scheme_make_hash_table)(int type);
 Scheme_Hash_Table *(*scheme_make_hash_table_equal)();
 Scheme_Hash_Table *(*scheme_make_hash_table_eqv)();
@@ -402,8 +408,9 @@ int (*scheme_is_hash_table_equal)(Scheme_Object *o);
 int (*scheme_is_hash_table_eqv)(Scheme_Object *o);
 Scheme_Hash_Table *(*scheme_clone_hash_table)(Scheme_Hash_Table *ht);
 void (*scheme_clear_hash_table)(Scheme_Hash_Table *ht);
+int (*scheme_hash_table_index)(Scheme_Hash_Table *hash, mzlonglong pos, Scheme_Object **_key, Scheme_Object **_val);
+Scheme_Object *(*scheme_hash_table_next)(Scheme_Hash_Table *hash, mzlonglong start);
 Scheme_Hash_Tree *(*scheme_make_hash_tree)(int kind);
-Scheme_Hash_Tree *(*scheme_make_hash_tree_set)(int kind);
 Scheme_Hash_Tree *(*scheme_hash_tree_set)(Scheme_Hash_Tree *tree, Scheme_Object *key, Scheme_Object *val);
 Scheme_Object *(*scheme_hash_tree_get)(Scheme_Hash_Tree *tree, Scheme_Object *key);
 Scheme_Object *(*scheme_eq_hash_tree_get)(Scheme_Hash_Tree *tree, Scheme_Object *key);
@@ -800,10 +807,6 @@ Scheme_Object *(*scheme_make_sized_path)(char *chars, intptr_t len, int copy);
 Scheme_Object *(*scheme_make_sized_offset_path)(char *chars, intptr_t d, intptr_t len, int copy);
 Scheme_Object *(*scheme_make_sized_offset_kind_path)(char *chars, intptr_t d, intptr_t len, int copy, int kind);
 Scheme_Object *(*scheme_make_path_without_copying)(char *chars);
-#ifdef MACINTOSH_EVENTS
-char *(*scheme_mac_spec_to_path)(mzFSSpec *spec);
-int (*scheme_mac_path_to_spec)(const char *filename, mzFSSpec *spec);
-#endif
 void *(*scheme_alloc_fdset_array)(int count, int permanent);
 void *(*scheme_init_fdset_array)(void *fdarray, int count);
 void *(*scheme_get_fdset)(void *fdarray, int pos);
@@ -1002,6 +1005,7 @@ Scheme_Object *(*scheme_malloc_key)(void);
 void (*scheme_free_key)(Scheme_Object *k);
 void *(*scheme_jit_find_code_end)(void *p);
 void (*scheme_jit_now)(Scheme_Object *f);
+void (*scheme_set_signal_handler)(int sig_id, Scheme_Signal_Handler_Proc proc);
 #ifndef SCHEME_EX_INLINE
 } Scheme_Extension_Table;
 #endif
