@@ -86,6 +86,7 @@
 #define scheme_add_flush (scheme_extension_table->scheme_add_flush)
 #define scheme_remove_flush (scheme_extension_table->scheme_remove_flush)
 #define scheme_add_atexit_closer (scheme_extension_table->scheme_add_atexit_closer)
+#define scheme_atexit (scheme_extension_table->scheme_atexit)
 #define scheme_add_evt (scheme_extension_table->scheme_add_evt)
 #define scheme_add_evt_through_sema (scheme_extension_table->scheme_add_evt_through_sema)
 #define scheme_is_evt (scheme_extension_table->scheme_is_evt)
@@ -118,6 +119,7 @@
 #define scheme_log_abort (scheme_extension_table->scheme_log_abort)
 #define scheme_log_warning (scheme_extension_table->scheme_log_warning)
 #define scheme_glib_log_message (scheme_extension_table->scheme_glib_log_message)
+#define scheme_glib_log_message_test (scheme_extension_table->scheme_glib_log_message_test)
 #define scheme_out_of_memory_abort (scheme_extension_table->scheme_out_of_memory_abort)
 #define scheme_wrong_count (scheme_extension_table->scheme_wrong_count)
 #define scheme_wrong_count_m (scheme_extension_table->scheme_wrong_count_m)
@@ -263,6 +265,7 @@
 #define scheme_collect_garbage (scheme_extension_table->scheme_collect_garbage)
 #define scheme_collect_garbage_minor (scheme_extension_table->scheme_collect_garbage_minor)
 #define scheme_enable_garbage_collection (scheme_extension_table->scheme_enable_garbage_collection)
+#define scheme_incremental_garbage_collection (scheme_extension_table->scheme_incremental_garbage_collection)
 #ifdef MZ_PRECISE_GC
 # ifndef USE_THREAD_LOCAL
 #define GC_variable_stack (scheme_extension_table->GC_variable_stack)
@@ -277,6 +280,7 @@
 #define scheme_free_immobile_box (scheme_extension_table->scheme_free_immobile_box)
 #define scheme_add_gc_callback (scheme_extension_table->scheme_add_gc_callback)
 #define scheme_remove_gc_callback (scheme_extension_table->scheme_remove_gc_callback)
+#define scheme_register_type_gc_shape (scheme_extension_table->scheme_register_type_gc_shape)
 #define scheme_make_bucket_table (scheme_extension_table->scheme_make_bucket_table)
 #define scheme_add_to_table (scheme_extension_table->scheme_add_to_table)
 #define scheme_change_in_table (scheme_extension_table->scheme_change_in_table)
@@ -285,6 +289,8 @@
 #define scheme_bucket_table_equal (scheme_extension_table->scheme_bucket_table_equal)
 #define scheme_clone_bucket_table (scheme_extension_table->scheme_clone_bucket_table)
 #define scheme_clear_bucket_table (scheme_extension_table->scheme_clear_bucket_table)
+#define scheme_bucket_table_index (scheme_extension_table->scheme_bucket_table_index)
+#define scheme_bucket_table_next (scheme_extension_table->scheme_bucket_table_next)
 #define scheme_make_hash_table (scheme_extension_table->scheme_make_hash_table)
 #define scheme_make_hash_table_equal (scheme_extension_table->scheme_make_hash_table_equal)
 #define scheme_make_hash_table_eqv (scheme_extension_table->scheme_make_hash_table_eqv)
@@ -298,8 +304,9 @@
 #define scheme_is_hash_table_eqv (scheme_extension_table->scheme_is_hash_table_eqv)
 #define scheme_clone_hash_table (scheme_extension_table->scheme_clone_hash_table)
 #define scheme_clear_hash_table (scheme_extension_table->scheme_clear_hash_table)
+#define scheme_hash_table_index (scheme_extension_table->scheme_hash_table_index)
+#define scheme_hash_table_next (scheme_extension_table->scheme_hash_table_next)
 #define scheme_make_hash_tree (scheme_extension_table->scheme_make_hash_tree)
-#define scheme_make_hash_tree_set (scheme_extension_table->scheme_make_hash_tree_set)
 #define scheme_hash_tree_set (scheme_extension_table->scheme_hash_tree_set)
 #define scheme_hash_tree_get (scheme_extension_table->scheme_hash_tree_get)
 #define scheme_eq_hash_tree_get (scheme_extension_table->scheme_eq_hash_tree_get)
@@ -574,10 +581,6 @@
 #define scheme_make_sized_offset_path (scheme_extension_table->scheme_make_sized_offset_path)
 #define scheme_make_sized_offset_kind_path (scheme_extension_table->scheme_make_sized_offset_kind_path)
 #define scheme_make_path_without_copying (scheme_extension_table->scheme_make_path_without_copying)
-#ifdef MACINTOSH_EVENTS
-#define scheme_mac_spec_to_path (scheme_extension_table->scheme_mac_spec_to_path)
-#define scheme_mac_path_to_spec (scheme_extension_table->scheme_mac_path_to_spec)
-#endif
 #define scheme_alloc_fdset_array (scheme_extension_table->scheme_alloc_fdset_array)
 #define scheme_init_fdset_array (scheme_extension_table->scheme_init_fdset_array)
 #define scheme_get_fdset (scheme_extension_table->scheme_get_fdset)
@@ -726,6 +729,7 @@
 #define scheme_free_key (scheme_extension_table->scheme_free_key)
 #define scheme_jit_find_code_end (scheme_extension_table->scheme_jit_find_code_end)
 #define scheme_jit_now (scheme_extension_table->scheme_jit_now)
+#define scheme_set_signal_handler (scheme_extension_table->scheme_set_signal_handler)
 #ifdef MZ_PRECISE_GC
 #pragma GC_VARIABLE_STACK_THOUGH_TABLE
 #endif
