@@ -327,6 +327,8 @@ scheme_init_type ()
 
   set_name(scheme_environment_variables_type, "<environment-variables>");
 
+  set_name(scheme_unquoted_printing_string_type, "<unquoted-printing-string>");
+
 #ifdef MZ_GC_BACKTRACE
   set_name(scheme_rt_runstack, "<runstack>");
   set_name(scheme_rt_meta_cont, "<meta-continuation>");
@@ -764,6 +766,8 @@ void scheme_register_traversers(void)
   GC_REG_TRAV(scheme_syntax_property_preserve_type, small_object);
 
   GC_REG_TRAV(scheme_plumber_handle_type, twoptr_obj);
+
+  GC_REG_TRAV(scheme_unquoted_printing_string_type, small_object);
 }
 
 END_XFORM_SKIP;
